#include "resumptiontoken.h"
#include "oaitools.h"

/**
 *  Generate an initial resumption token from some basic details.
 *
 *  TODO: add optional argument to set the server name.
 */
ResumptionToken::ResumptionToken(const text_t &collection, const text_t &node,
				 const text_t &buildDate)
{ this->collection = collection;
  this->browseNode = node;
  this->buildDate  = buildDate;
  this->startItem  = 0;
}

/**
 *  Generate a resumption token from a URN-style format.
 *
 *  See getToken() for details of the format.
 *
 *  TODO: support inclusion of an optional server name.
 */
ResumptionToken::ResumptionToken(const text_t &URN)
{ text_t::const_iterator first = URN.begin();
  text_t::const_iterator last = URN.end(); 

  this->collection = "";
  this->browseNode = "";
  this->startItem  = -1;
  
  text_t::const_iterator here = findchar(first, last, ':');
  if (here == first) {
    return;
  }
  
  text_t oainamespace = substr(first, here);
  if (oainamespace != "gsdloai") {
    return;
  }

  // increment past the first colon to get the location
  first = ++here;

  // get the collection
  here = findchar(first, last, '.');
  if (here == first) {
    return;
  }
  this->collection = substr(first, here);

  first = ++here;

  // get the browse node
  here = findchar(first, last, ',');

  // now, we *may* have truncated items where we start with the first
  // item in a node (i.e. suppress the ',0' for one reason or another,
  // let's be forgiving if we can...
  if (here == first) {
    this->startItem = 0;

    here = findchar(first, last, '-');    

    // no build date - keep trying...
    if (here == first) {
      this->browseNode = substr(first, last);
    }
    else {
      this->browseNode = substr(first, here);
      this->buildDate  = substr(++here, last);
    }
  }
  // got a proper item indicator...
  else {
    // record the browse node
    this->browseNode = substr(first, here);

    // get past the ','
    first = ++here;
    
    // now find the build date marker
    here = findchar(first, last, '-');
    if (here == first) {
      this->startItem = substr(first, last).getint();
    }
    else {
      this->startItem = substr(first, here).getint();
      this->buildDate = substr(++here, last); 
    }
  }
}

/**
 *  Get a resumption token in text_t format.
 * 
 *  Resumption tokens are in the format:
 *
 *    gsdloai:<serverName>:collectionname.browseNode,startItem-BuildDate
 *
 *  The resumption token format does not currently implement the use of
 *  the optional <serverName> item; it is taken to default to the name of
 *  the receiving server.
 *
 *  TODO: add server identity as an optional argument; also change
 *        ResumptionToken(text_t &) accordingly.
 */
text_t ResumptionToken::getToken()
{ text_t reply = "gsdloai:";
  reply = reply + this->collection;
  if (this->browseNode != "") {
    reply = reply + "." + this->browseNode;
  }
  reply = reply + ",";
  reply.append(this->startItem);
  reply = reply + "-" + buildDate;

  return reply;
}

/**
 *  Update the position of an existing resumption token
 */
void ResumptionToken::setPosition(const text_t &node, int startItem)
{ this->browseNode = node;
  this->startItem  = startItem; 
}

/**
 *  Check if the resumption token is valid - only a very primitive
 *  check is done here; one ought to check for an existing collection
 *  and valid browse Node, build date and startItem
 *
 *  TODO: implement improved validation checking.
 */
bool ResumptionToken::isValid()
{ return this->collection != "";
}
